"""DROID circuits. These circuits are auto-generated from circuits.json."""

from dataclasses import dataclass, field
from typing import Optional

from metapatch.circuits.base import DroidCircuit
from metapatch.circuits import circuit_types as ctype


__droid_version__ = "magenta-3"


@dataclass
class Calibrator(DroidCircuit):
    """VCO Calibrator.

    Args:
        input (voltperoctave):
          Patch your V/Oct pitch input here.
        nudgeup (trigger):
          A trigger here (most likely a button press) will modify the tuning of the
          currently played note (as read by input) upwards by one cent (or by
          nudgeamount if that is used.
        nudgedown (trigger):
          A trigger here will modify the tuning of the currently played note down.
        clearhere (trigger):
          A trigger here sets the correction of the currently played note to zero. This
          might affect a range of up to two octaves.
        nudgeamount (cv):
          Changes the amount each button press detunes. A value of one would mean one
          semitone, so the default value of 0.01 corresponds to one cent (1/100) of a
          semitone.
        tune0 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune1 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune2 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune3 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune4 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune5 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune6 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune7 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tune8 (cv):
          Explicit tuning of the octaves 0 through 8 – if you do not want to nudge
          manually. tune0 sets the tuning for the input pitch of 0 V, tune1 for 1 V and
          so on. A value of 1 means a tune adjustment of one semitone – which is 100
          cent. The maximum detuning is ± 1 Octave ( at a value of ± 12).
        tunelowtail (cv):
          Tuning adaption for the negative voltage range. A value of 1 means an upwards
          tuning of one semitone per octave, -1 likewise downwards.
        tunehightail (cv):
          Tuning adaption for voltages > 8 V. A value of 1 means an upwards tuning of
          one semitone per octave, -1 likewise downwards.
        select (integer):
          The select input allows you to overlay buttons and LEDs with multiple
          functions. If you use this input, the circuit will process the buttons and
          LEDs just as if select has a positive gate signal (usually you will select
          this to 1). Otherwise it won't touch them so they can be used by another
          circuit. Note: even if the circuit is currently not selected, it will
          nevertheless work and process all its other inputs and its  outputs (those
          that do not deal with buttons or LEDs) in a normal way.
        selectat (integer):
          This input makes the select input more flexible. Here you specify at which
          value select should select this circuit. E.g. if selectat is 0, the circuit
          will be active if select is exactly 0 instead of a positive gate signal. In
          some cases this is more conventient.
        preset (integer):
          This is the preset number to save or to load. Note: the first preset has the
          number 0, not 1! For the whole story on presets please refer to page 21.
        loadpreset (trigger):
          A trigger here loads a preset. As a speciality you can use the trigger for
          selecting a preset at the same time.
        savepreset (trigger):
          A trigger here saves a preset.
        clear (trigger):
          A trigger here loads the default start state into the circuit. The presets are
          not affected, unless you use direct preset switching with the preset input and
          without triggers. And that case the current preset is also cleared.
        clearall (trigger):
          A trigger here loads the default start state into the circuit and into all of
          its presets.
        dontsave (gate):
          If you set this to 1, the state of the circuit will not saved to the SD card
          and not loaded from the SD card when the Droid starts.
        output (voltperoctave):
          The calibrated pitch goes out here.
        ledup (fraction):
          When nudgeup is mapped to a button (which is most likely), map this output to
          the according LED and it will indicate whenever it's currently adjusting the
          output pitch upwards.
        leddown (fraction):
          This is the LED for nudgedown, which indicates downwards adjustment.
        correction (cv):
          This output gives you information about the current amout of pitch correction.
          It is positive if the pitch is corrected upwards, else negative. It is scaled
          in semitones, so a value of 0.2 means a 20% of a semitone, which is the same
          is 20 cents.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 224
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    nudgeup: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    nudgedown: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    clearhere: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    nudgeamount: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune0: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tune8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tunelowtail: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tunehightail: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    select: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    selectat: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    preset: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    loadpreset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    savepreset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    clear: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    clearall: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    dontsave: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    ledup: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    leddown: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    correction: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Chord(DroidCircuit):
    """Chord generator.

    Args:
        pitch (voltperoctave):
          This sets the minimum pitch of the lowest note of the chord.
        spread (voltperoctave):
          Selects the range between the lowest and highest note of the chord measured in
          1V/oct, while a spread of 0 means that all chord notes are within one octave,
          a spread of 1 V means that the notes are spread out over two octaves and so
          on.
        inversion (integer):
          Selects the inversion of the chord. 1 means that the root note should be the
          lowest note, 2 will make the second selected note the lowest note, 3 the 3 and
          4 the 4. The default, however, is 0 and doesn't fix the inversion. Rather that
          inversion is chosen that creates the chord closest to the input pitch.
        trigger (trigger):
          This jack is optional. If you patch it, the Chord generator just reads a new
          input pitch when it receives a trigger.
        root (integer):
          Set the root note here. 0 means C, 1 means C♯, 2 means D and so on. If you
          multiply the value of an input like I1 with 120, then you can use a 1V/Oct
          input for selecting the root note via a sequencer, MIDI keyboard or the like.
          Also then you are compatible with the ROOT CV input of the Sinfonion.   0C 1C♯
          2D 3D♯ 4E 5F 6F♯ 7G 8G♯ 9A 10A♯ 11B 12C
        degree (integer):
          Set the musical scale. This is a number from 0 to 107. Below are the first 12
          and most important scales. You find a list of all 108 scales on page 106.
          0lyd – Lydian major scale (it has a ♯ 4) 1maj – Normal major scale (ionian)
          2X^7 – Mixolydian (dominant seven chords) 3sus – mixolydian with 3/4 swapped
          4alt – Altered scale 5hm^5 – Harmonic minor scale from the 5 6dor – Dorian
          minor (minor with ♯ 13) 7min – Natural minor (aeolian) 8hm – Harmonic minor (♭
          6 but ♯ 7) 9phr – Phrygian minor scale (with ♭ 9) 10dim – Diminished scale
          (whole/half tone) 11aug – Augmented scale (just whole tones)   Note:
          Alltogether there are 108 scales. Please see page 106 for a complete list
        select1 (gate):
          Gate input for selecting the root note as being an allowed interval. When you
          want to create a playing interface for live operation you can patch the output
          of a toggle button (made with the circuit [button]) here.  Note: When all
          select and selectfill inputs are 0, automatically all seven scale notes are
          selected, i.e. select1 ... select13 will be set to one.
        select3 (gate):
          Gate input for selecting the 3.
        select5 (gate):
          Gate input for selecting the 5.
        select7 (gate):
          Gate input for selecting the 7.
        select9 (gate):
          Gate input for selecting the 9 (which is the same as the 2).
        select11 (gate):
          Gate input for selecting the 11 (which is the same as the 4).
        select13 (gate):
          Gate input for selecting the 13 (which is the same as the 6).
        selectfill1 (gate):
          Selects the alternative 9 (i.e. the 9 that is not in the scale.
        selectfill2 (gate):
          Selects the alternative 3 (i.e. the 3 that is not in the scale).
        selectfill3 (gate):
          Selects the alternative 4 or 5. In most cases this is the diminished 5.
        selectfill4 (gate):
          Selects the alternative 13 (i.e. the 13 that is not in the scale).
        selectfill5 (gate):
          Selects the alternative 7 (i.e. the 7 that is not in the scale).
        harmonicshift (integer):
          This input can reduce harmonic complexity by disabling some of the scale or
          non-scale notes. It is an idea first found in the Sinfonion and also provided
          by the circuit sinfonionlink.  harmonicshift is staged after the select...
          inputs and further filters out (disables) notes based on their relation to the
          current scale. This means that first the 12 select... inputs select a subset
          of the 12 possible notes. After that harmonicshift can reduce this set further
          (it will never add notes).  If harmonicshift is not zero, depending on its
          value some or more of the scale notes are disabled, even if they would be
          allowed by select....  Or in other words: the harmonic material is reduced.
          You also can use negative values. These create rather strange sounds by
          removing the simple chord functions instead of the complex ones first.  Here
          are the possible values:   0off – all selected notes are allowed 1disable all
          fill notes (non-scale notes) 2disable fills and 11þ 3disable fills, 11þand 13þ
          4disable fills, 11þ, 13þand 9 5disable fills, 11þ, 13þ, 9 and 7 6disable
          fills, 11þ, 13þ, 9, 7 and 3 7disable fills, 11þ, 13þ, 9, 7, 3 and 5 -1disable
          the root note -2disable the root note and the 5 -3disable root, 3, 5 -4disable
          root, 3, 5, 7 -5disable root, 3, 5, 7, 9 -6disable root, 3, 5, 7, 9 and 13þ
          -7disable all scale notes (fill notes untouched)
        noteshift (integer):
          Shifts the resulting output note(s) by this number of scale notes up or down
          (if negative). So the output note still is part of the scale but may be a note
          that is none of the selected ones. The maximum shift range is limited to -24 …
          +24.
        selectnoteshift (integer):
          Shifts the output note by this number of selected scale notes up or down (if
          negative). If you use noteshift at the same time, first selectnoteshift is
          applied, then noteshift. The maximum shift range is limited to -24 … +24.
        tuningmode (gate):
          While this is 1, the circuit will output the value set by tuningpitch instead
          of the actual pitch. This is ment to be a help for tuning your VCOs.
        tuningpitch (voltperoctave):
          This pitch CV will be output while the tuning mode is active.
        transpose (voltperoctave):
          This value is being added to the output pitch when not in tuning mode. It can
          be used for musical transposition or adding a vibrato.
        output1 (voltperoctave):
          1 ... 4 pitch output
        output2 (voltperoctave):
          1 ... 4 pitch output
        output3 (voltperoctave):
          1 ... 4 pitch output
        output4 (voltperoctave):
          1 ... 4 pitch output
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 136
    pitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    spread: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    inversion: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    trigger: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    root: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    degree: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    select1: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select3: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select5: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select7: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select9: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select11: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select13: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill1: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill2: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill3: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill4: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill5: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    harmonicshift: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    noteshift: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    selectnoteshift: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    tuningmode: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    tuningpitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    transpose: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output3: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output4: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )



@dataclass
class Detune(DroidCircuit):
    """Detune multiple voices in a most disharmonic way.

    Args:
        input1 (cv):

        input2 (cv):

        input3 (cv):

        input4 (cv):

        input5 (cv):

        input6 (cv):

        input7 (cv):

        input8 (cv):

        detune (cv):

        tuningmode (gate):
          While this is 1, the circuit will output the value set by tuningpitch instead
          of the actual pitch. This is ment to be a help for tuning your VCOs.
        tuningpitch (voltperoctave):
          This pitch CV will be output while the tuning mode is active.
        output1 (cv):

        output2 (cv):

        output3 (cv):

        output4 (cv):

        output5 (cv):

        output6 (cv):

        output7 (cv):

        output8 (cv):

        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 56
    input1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    detune: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    tuningmode: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    tuningpitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Fold(DroidCircuit):
    """CV folder – keep (pitch) CV within certain bounds.

    Args:
        input (cv):
          Input CV to be folded.
        foldby (cv):
          Amount to be added or substracted from the input CV if it is not within the
          allowed range. This CV must be positive. If it is negative or zero, no folding
          will be done.
        minimum (cv):
          Lower bound of the allowed range. If unpatched, no lower bound will be
          applied.
        maximum (cv):
          Upper bound of the allowed range. If unpatched, no upper bound will be
          applied.
        output (cv):
          Folded output voltage
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 32
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    foldby: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    minimum: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    maximum: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Minifonion(DroidCircuit):
    """Musical quantizer.

    Args:
        input (voltperoctave):
          Patch the unquantized input voltage here
        trigger (trigger):
          This jack is optional. If you patch it, the Minifonion will work in triggered
          mode. Here the output pitch is always frozen until the next trigger happens.
        bypass (gate):
          If you set this gate input to 1 then quantization is bypassed and the input
          voltage is directly copied to the output.
        root (integer):
          Set the root note here. 0 means C, 1 means C♯, 2 means D and so on. If you
          multiply the value of an input like I1 with 120, then you can use a 1V/Oct
          input for selecting the root note via a sequencer, MIDI keyboard or the like.
          Also then you are compatible with the ROOT CV input of the Sinfonion.   0C 1C♯
          2D 3D♯ 4E 5F 6F♯ 7G 8G♯ 9A 10A♯ 11B 12C
        degree (integer):
          Set the musical scale. This is a number from 0 to 107. Below are the first 12
          and most important scales. You find a list of all 108 scales on page 106.
          0lyd – Lydian major scale (it has a ♯ 4) 1maj – Normal major scale (ionian)
          2X^7 – Mixolydian (dominant seven chords) 3sus – mixolydian with 3/4 swapped
          4alt – Altered scale 5hm^5 – Harmonic minor scale from the 5 6dor – Dorian
          minor (minor with ♯ 13) 7min – Natural minor (aeolian) 8hm – Harmonic minor (♭
          6 but ♯ 7) 9phr – Phrygian minor scale (with ♭ 9) 10dim – Diminished scale
          (whole/half tone) 11aug – Augmented scale (just whole tones)   Note:
          Alltogether there are 108 scales. Please see page 106 for a complete list
        select1 (gate):
          Gate input for selecting the root note as being an allowed interval. When you
          want to create a playing interface for live operation you can patch the output
          of a toggle button (made with the circuit [button]) here.  Note: When all
          select and selectfill inputs are 0, automatically all seven scale notes are
          selected, i.e. select1 ... select13 will be set to one.
        select3 (gate):
          Gate input for selecting the 3.
        select5 (gate):
          Gate input for selecting the 5.
        select7 (gate):
          Gate input for selecting the 7.
        select9 (gate):
          Gate input for selecting the 9 (which is the same as the 2).
        select11 (gate):
          Gate input for selecting the 11 (which is the same as the 4).
        select13 (gate):
          Gate input for selecting the 13 (which is the same as the 6).
        selectfill1 (gate):
          Selects the alternative 9 (i.e. the 9 that is not in the scale.
        selectfill2 (gate):
          Selects the alternative 3 (i.e. the 3 that is not in the scale).
        selectfill3 (gate):
          Selects the alternative 4 or 5. In most cases this is the diminished 5.
        selectfill4 (gate):
          Selects the alternative 13 (i.e. the 13 that is not in the scale).
        selectfill5 (gate):
          Selects the alternative 7 (i.e. the 7 that is not in the scale).
        harmonicshift (integer):
          This input can reduce harmonic complexity by disabling some of the scale or
          non-scale notes. It is an idea first found in the Sinfonion and also provided
          by the circuit sinfonionlink.  harmonicshift is staged after the select...
          inputs and further filters out (disables) notes based on their relation to the
          current scale. This means that first the 12 select... inputs select a subset
          of the 12 possible notes. After that harmonicshift can reduce this set further
          (it will never add notes).  If harmonicshift is not zero, depending on its
          value some or more of the scale notes are disabled, even if they would be
          allowed by select....  Or in other words: the harmonic material is reduced.
          You also can use negative values. These create rather strange sounds by
          removing the simple chord functions instead of the complex ones first.  Here
          are the possible values:   0off – all selected notes are allowed 1disable all
          fill notes (non-scale notes) 2disable fills and 11þ 3disable fills, 11þand 13þ
          4disable fills, 11þ, 13þand 9 5disable fills, 11þ, 13þ, 9 and 7 6disable
          fills, 11þ, 13þ, 9, 7 and 3 7disable fills, 11þ, 13þ, 9, 7, 3 and 5 -1disable
          the root note -2disable the root note and the 5 -3disable root, 3, 5 -4disable
          root, 3, 5, 7 -5disable root, 3, 5, 7, 9 -6disable root, 3, 5, 7, 9 and 13þ
          -7disable all scale notes (fill notes untouched)
        noteshift (integer):
          Shifts the resulting output note(s) by this number of scale notes up or down
          (if negative). So the output note still is part of the scale but may be a note
          that is none of the selected ones. The maximum shift range is limited to -24 …
          +24.
        selectnoteshift (integer):
          Shifts the output note by this number of selected scale notes up or down (if
          negative). If you use noteshift at the same time, first selectnoteshift is
          applied, then noteshift. The maximum shift range is limited to -24 … +24.
        tuningmode (gate):
          While this is 1, the circuit will output the value set by tuningpitch instead
          of the actual pitch. This is ment to be a help for tuning your VCOs.
        tuningpitch (voltperoctave):
          This pitch CV will be output while the tuning mode is active.
        transpose (voltperoctave):
          This value is being added to the output pitch when not in tuning mode. It can
          be used for musical transposition or adding a vibrato.
        output (voltperoctave):
          Here comes your quantized output voltage
        notechange (trigger):
          Whenever the quantization changes to a new note a trigger with the duration
          10 ms is output here. No trigger is output in bypass mode.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 112
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    trigger: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    bypass: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    root: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    degree: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    select1: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select3: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select5: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select7: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select9: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select11: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select13: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill1: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill2: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill3: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill4: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    selectfill5: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    harmonicshift: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    noteshift: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    selectnoteshift: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    tuningmode: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    tuningpitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    transpose: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    notechange: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )



@dataclass
class Octave(DroidCircuit):
    """Multi-VCO octave animator.

    Args:
        input (voltperoctave):
          The general pitch information on a 1 V / octave base to be used for the three
          VCOs.
        spread (stepped):
          The amount of octave spread between output1 and output3. At a value of 1.0 the
          spread is four octaves.
        detune (fraction):
          The amount of linear detune of VCO 2 and 3. This is not on a 1 V / octave base
          but corresponds to an absolute frequency difference in Hertz. The exact
          frequency difference cannot be set here, since that depends on how you have
          tuned your VCOs. But the rule is the following: If input is a 0 V and detune
          is 1.0, the detune is by four semitones. And for an input of 1 V (one octave
          higher) it is just two semitones, because that results in the same frequency
          difference. For 2 V (two octaves up) it ist just one semitone and for 3 V half
          a semitone (and so on). Best thing is to simply try out and listen!
        fifths (gate):
          Set this to 1 or on if you want to include perfect fifths as intermediate
          steps.
        output1 (voltperoctave):
          Outputs for the 1 V / octave of the three VCOs. output1 is an exact copy of
          input so you could omit that and rather patch VCO 1 to the original pitch CV.
        output2 (voltperoctave):
          Outputs for the 1 V / octave of the three VCOs. output1 is an exact copy of
          input so you could omit that and rather patch VCO 1 to the original pitch CV.
        output3 (voltperoctave):
          Outputs for the 1 V / octave of the three VCOs. output1 is an exact copy of
          input so you could omit that and rather patch VCO 1 to the original pitch CV.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 32
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    spread: Optional[str] = field(
            default=None,
            metadata=ctype.type_stepped(ramsize=0)
    )
    detune: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    fifths: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output3: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )



@dataclass
class Sinfonionlink(DroidCircuit):
    """Sync harmonic state from Sinfonion.

    Args:
        root (integer):
          The current root note as an integer number. C = 0, C♯ = 1, D = 2 and so on.
        degree (integer):
          The current scale (the Sinfonion uses the word degree for this). This is an
          integer number. If find a list of all available scales on page 106.
        transpose (voltperoctave):
          The current global transposition of the Sinfonion. This is in 1V/Oct, so you
          can add it to your pitch whereever you output one.
        chaoticdetune (cv):
          The current value of the chaotic detune. You can feed this into the detune
          input of the circuit detune.
        harmonicshift (integer):
          Harmonic shift is a feature of the Sinfonion that allows to reduce harmonic
          complexity via CV (or the builtin pots POT1 or POT2). The idea is that the
          more you rise the CV, the less complex scale notes are allowed.  This output
          gives you access to the current setting of harmonic shift of the Sinfonion. It
          is an integer number between -7 and 7. You can directly feed it into the
          harmonicshift input of circuits like minifonion, chord, arpeggio or
          motoquencer. Harmonic shift is explained in detail in the manual chapter of
          minifonion.
        linkstate (gate):
          Outputs 1 if the link to the Sinfonion is up and active, otherwise 0.
        clock (trigger):
          Gives you a copy of the Sinfonion's clock input
        reset (trigger):
          Outputs a trigger whenever in Song mode the Sinfonion forwards to the first
          bar of the song.
        step (trigger):
          Outputs a trigger for every step of a song.
        beat (trigger):
          Outputs a trigger for every beat (subdivision of a step).
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 56
    root: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    degree: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    transpose: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    chaoticdetune: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    harmonicshift: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    linkstate: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    reset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    step: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    beat: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )



@dataclass
class Superjust(DroidCircuit):
    """Perfect intonation of up to eight voices.

    Args:
        input1 (voltperoctave):
          1 ... 8 pitch input
        input2 (voltperoctave):
          1 ... 8 pitch input
        input3 (voltperoctave):
          1 ... 8 pitch input
        input4 (voltperoctave):
          1 ... 8 pitch input
        input5 (voltperoctave):
          1 ... 8 pitch input
        input6 (voltperoctave):
          1 ... 8 pitch input
        input7 (voltperoctave):
          1 ... 8 pitch input
        input8 (voltperoctave):
          1 ... 8 pitch input
        tuningmode (gate):
          While this is 1, all outputs output the value set by tuningpitch. This is for
          tuning all outputs. Since perfect tuning is crucial for perfect intonation,
          this is quite useful.
        tuningpitch (voltperoctave):
          This pitch CV will be output while the tuning mode is active.
        bypass (gate):
          While this is 1, all inputs are passed through to the outputs without changes.
        transpose (voltperoctave):
          This value is being added to all outputs, but not in tuning or bypass mode. It
          can e.g. be used for making a vibrato on a chord.
        output1 (voltperoctave):
          1 ... 8 pitch output
        output2 (voltperoctave):
          1 ... 8 pitch output
        output3 (voltperoctave):
          1 ... 8 pitch output
        output4 (voltperoctave):
          1 ... 8 pitch output
        output5 (voltperoctave):
          1 ... 8 pitch output
        output6 (voltperoctave):
          1 ... 8 pitch output
        output7 (voltperoctave):
          1 ... 8 pitch output
        output8 (voltperoctave):
          1 ... 8 pitch output
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 64
    input1: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    input2: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    input3: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    input4: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    input5: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    input6: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    input7: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    input8: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    tuningmode: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    tuningpitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    bypass: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    transpose: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output3: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output4: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output5: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output6: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output7: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    output8: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )



@dataclass
class Vcotuner(DroidCircuit):
    """measure frequency and tuning of a VCO.

    Args:
        tuningnote (integer):
          You can either tune to a specific note, like C or A or whatever you like. In
          this case set tuningnote to a number from 0 to 11, where 0 is C, 1 is C#, 2 is
          D and so on. Or you set it to -1 (which is the default) to tune to the nearest
          semitone.
        concertpitch (cv):
          Set the frequency of the concert pitch here, which is the frequency of A1.
          Most commonly this is 440 Hz, so 440 is the default value of this parameter.
        basepitch (voltperoctave):
          This output sets the reference for the pitch output. It is the voltage for the
          note C0. With analog synths it is common – but not neccessary – that the C
          notes correspond to integer voltage numbers.
        smooth (fraction):
          When measuring the input frequency, a slight smoothing is applied (like a low
          pass filter). It evens out some possible jitter that you might have. You can
          adjust this filter here. Per default smoothing is just subtle. Beware: smooth
          = 1 smoothes strong, so the frequency measurement needs some time to get
          accurate.
        precision (cv):
          This parameter defines the precision for the input frequency to be in tune. It
          is set in cents, where 100 cent correspond to one semitone. If you set the
          precision too low, it is very hard to tune correctly. If you set it too high,
          your tuning is not very precise.
        select (integer):
          The select input allows you to overlay buttons and LEDs with multiple
          functions. If you use this input, the circuit will process the buttons and
          LEDs just as if select has a positive gate signal (usually you will select
          this to 1). Otherwise it won't touch them so they can be used by another
          circuit. Note: even if the circuit is currently not selected, it will
          nevertheless work and process all its other inputs and its  outputs (those
          that do not deal with buttons or LEDs) in a normal way.
        selectat (integer):
          This input makes the select input more flexible. Here you specify at which
          value select should select this circuit. E.g. if selectat is 0, the circuit
          will be active if select is exactly 0 instead of a positive gate signal. In
          some cases this is more conventient.
        hz (cv):
          Outputs the current input frequency. If you no signal is found, the output is
          0.
        ledflat (fraction):
          Patch this to a LED to show if and how much the input frequency is flat (too
          low). This value ranges from 0.2 to 1.0 if the input frequency is flat and
          goes straight to 0 otherwise. If you use select, this output only works while
          the circuit is selected.
        ledsharp (fraction):
          Patch this to a LED to show if and how much the input frequency is sharp (too
          high). This value ranges from 0.2 to 1.0 if the input frequency is sharp and
          goes straight to 0 otherwise. If you use select, this output only works while
          the circuit is selected.
        ledintune (gate):
          Outputs 1 if the input frequency is “in tune”. Here the precision input is
          applied. If you use select, this output only works while the circuit is
          selected.
        intune (gate):
          Outputs 1 if the input frequency is “in tune”. Here the precision input is
          applied. This output also while while the circuit is not selected. You can use
          it in situations where you want to process this information by some other
          circuit rather than just displaying it.
        tuning (bipolar):
          Outputs the current tuning deviation of the input measured in 1V/octave.
        cents (bipolar):
          Outputs the current tuning deviation in cents, where 100 cents correspond to
          one semitone.
        pitch (voltperoctave):
          Outputs the current pitch of the input frequency in terms of 1V/octave. Here
          the basepitch is applied. You can patch this output to the pitch input of a
          second VCO to have that follow the pitch of the measured VCO.
        referencepitch (voltperoctave):
          Here you get the pitch of the reference tone the tuner currently “locked in”
          into. It is either the nearest semitone or the note selected by tuningnote in
          the nearest octave.
        vcofound (gate):
          This output is 1 whenever a valid input signal is found on the tuning input
          I1.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 0
    tuningnote: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    concertpitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    basepitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    smooth: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    precision: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    select: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    selectat: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    hz: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    ledflat: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    ledsharp: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    ledintune: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    intune: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    tuning: Optional[str] = field(
            default=None,
            metadata=ctype.type_bipolar(ramsize=0)
    )
    cents: Optional[str] = field(
            default=None,
            metadata=ctype.type_bipolar(ramsize=0)
    )
    pitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    referencepitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_voltperoctave(ramsize=0)
    )
    vcofound: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )






"""DROID circuits. These circuits are auto-generated from circuits.json."""

from dataclasses import dataclass, field
from typing import Optional

from metapatch.circuits.base import DroidCircuit
from metapatch.circuits import circuit_types as ctype


__droid_version__ = "magenta-3"


@dataclass
class Case(DroidCircuit):
    """Switch choosing from inputs via conditions.

    Args:
        case1 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case2 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case3 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case4 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case5 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case6 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case7 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case8 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case9 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case10 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case11 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case12 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case13 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case14 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case15 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        case16 (cv):
          1 ... 16 case input. The first one that is non-zero defines which input value
          to use.
        value1 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value2 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value3 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value4 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value5 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value6 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value7 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value8 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value9 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value10 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value11 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value12 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value13 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value14 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value15 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        value16 (cv):
          1 ... 16 value input. One of these is copied to the output, depending on which
          of the case inputs is none-zero.
        else_ (trigger):
          In case none of the case inputs is non-zero, this value is copied to the
          output.
        output (cv):
          To this output the select value input is copied.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 88
    case1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case9: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case10: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case11: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case12: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case13: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case14: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case15: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    case16: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value9: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value10: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value11: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value12: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value13: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value14: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value15: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    value16: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    else_: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Crossfader(DroidCircuit):
    """Morph between 8 inputs.

    Args:
        input1 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        input2 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        input3 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        input4 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        input5 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        input6 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        input7 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        input8 (cv):
          The input signals that you want to crossfade between. At least input1 and
          input2 need to be patched. Otherwise they are treated like 0 V signals.
        fade (fraction):
          This value decides which of the two inputs should be mixed and to which degree
          each one should go into the mix. At 0.0 the mix consists of 100% of the first
          inputs, at 1.0 of 100% of the last patched input.
        output (cv):
          Output of the mix
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 40
    input1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    fade: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Cvlooper(DroidCircuit):
    """Clocked CV looper.

    Args:
        cvin (cv):
          Input CV that should be looped.
        gatein (gate):
          Optional input gate. If you do not patch something here, the gate is assumed
          to be always high.
        clock (trigger):
          Input clock. The clock is mandatory and is the base for the definition of the
          loop length. Also the loop switch is quantized in time to the nearest clock.
        reset (trigger):
          A trigger here resets the playback head immediately to the start of the loop,
          if you are in playback mode.
        length (integer):
          Length of the loop in clock ticks. Example: You get a length of 16 ticks by
          patching the number 16 to length. If you want to set the length by means of an
          external CV that would require 160 Volts. So you need to multiply your input
          by some useful number in that case.
        tapespeed (cv):
          Relative tape speed, where 1.0 is the normal speed. So a value of 0.5 slows
          down the speed thus increasing the effective tape length from 8 to 16 seconds
          while reducing the sampling rate from 1 ms to 2 ms per sample. Changing the
          tape speed on the fly probably leads to interesting results.
        loopswitch (gate):
          Mandatory parameter: While the loop switch is off the CV looper simply sends
          all input CV and gate to their respective outputs. At the same time CV and
          gate are also recorded to the tape. When the loop switch is on, the CV and
          gate are being read from the tape, instead. The input CV and gate are now
          ignored.
        pause (gate):
          This is a binary input. If you send a high signal here, the looper pauses.
          This is only works in playback mode. The current CV value is hold the entire
          time. This is not the same as bypass, since in bypass mode the original CV
          will be routed through.
        overlay (gate):
          Overlaying changes the behaviour while looping is active. If overlay is set to
          on, while the input gate is active the gate and CV will be sent directly from
          the inputs rather than read from the tape.
        overdub (gate):
          Overdubbing also changes the behaviour during the looping: If it is active
          then while the input gate is high the input gate and CV will be written to the
          tape – thus changing the loop on the fly.
        bypass (gate):
          Setting bypass to on copies the input CV and gate from their inputs to their
          outputs while keeping the loop's content untouched. This disabled the looping
          for the while, but you can get back to it later. Note: this is different from
          turning off the loop switch, because then your tape's content would be
          overwritten.
        cvout (cv):
          Output of the bypassed or looped CV
        gateout (gate):
          Output of the bypassed or looped gate
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 17336
    cvin: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    reset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    length: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    tapespeed: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    loopswitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    pause: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    overlay: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    overdub: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    bypass: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    cvout: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gateout: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )



@dataclass
class Delay(DroidCircuit):
    """A tape delay for CVs, gates and numbers.

    Args:
        delay (cv):
          The CVs are delayed by this amount of seconds. If you patch clock as well, the
          delay is specified in clock tick, so then delay = 1 means “delay by one clock
          tick”.
        cvin (cv):
          Continous input CV
        numberin (integer):
          Discrete input number in the range 0 ... 255
        gatein1 (cv):
          Input gates
        gatein2 (cv):
          Input gates
        gatein3 (cv):
          Input gates
        gatein4 (cv):
          Input gates
        gatein5 (cv):
          Input gates
        gatein6 (cv):
          Input gates
        gatein7 (cv):
          Input gates
        gatein8 (cv):
          Input gates
        clock (trigger):
          If you use this clock input, all time inputs are measured in clock ticks
          instead of seconds.
        sample (gate):
          If you patch this input, “triggered” mode is enabled. In this mode, the
          virtual tape just records a new CV on each trigger at sample. So it just
          records stepped CVs, no slopes and no CV changes between the triggers.
        timewindow (cv):
          When in triggered mode, this optional parameter helps tackling a problem that
          many hardware sequencers show: often their pitch CV is not at its final
          destination value at the time their gate is being output. Often you see a very
          short “slew” ramp of say 5 ms after the gate. During that time the pitch CV
          moves from its former to the new value.  Now if you trigger the cvtape circuit
          with the sequencer's gate you will essentially sample the previous pitch CV
          instead of the new one. Or maybe something in between.  The timewindow
          parameter configures a short time window after the trigger to trigger. During
          that time period the tape will constantly adapt the last sample to a changed
          input CV. When that time is over, the input is finally frozen on the tape.
          The timewindow parameter is in seconds. So when you set timewindow to say
          0.005 (which means 5 ms), you give the input CV 5 ms time for settling to its
          final value after a trigger to sample before freezing it.
        bypass (gate):
          Setting bypass to on copies the input signals directly to the outputs,
          regardless of any other stuff going on.
        save (trigger):
          Send a trigger here to save the current contents of the tape to a file on the
          SD card. The filename is tapeXXXX.bin, where XXXX is replaced by the number
          set by filenumber.
        load (trigger):
          Send a trigger here to load a previously saved file into the tape. Use
          filenumber so specify which file to load.
        filenumber (integer):
          Number of the file to load or save. The range is 0 - 9999. If filenumber is
          123, the name on the SD card is tape0123.bin. These files are shared between
          all recorder and delay circuits.
        cvout (cv):
          Output of the continous input CV
        numberout (integer):
          Output of the discrete number
        gateout1 (gate):
          Output of the gates
        gateout2 (gate):
          Output of the gates
        gateout3 (gate):
          Output of the gates
        gateout4 (gate):
          Output of the gates
        gateout5 (gate):
          Output of the gates
        gateout6 (gate):
          Output of the gates
        gateout7 (gate):
          Output of the gates
        gateout8 (gate):
          Output of the gates
        overflow (gate):
          When the internal memory of the tape is exceeded and data got lost, this gate
          goes to 1 for 0.5 seconds. If you are suspecting this situation, you can wire
          this output to an LED and observe the memory status that way.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 1672
    delay: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    cvin: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    numberin: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    gatein1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    sample: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    timewindow: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    bypass: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    save: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    load: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    filenumber: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    cvout: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    numberout: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    gateout1: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout2: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout3: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout4: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout5: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout6: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout7: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout8: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    overflow: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )



@dataclass
class Matrixmixer(DroidCircuit):
    """Matrix mixer for CVs.

    Args:
        input1 (cv):
          The up to four CV inputs that you want to mix
        input2 (cv):
          The up to four CV inputs that you want to mix
        input3 (cv):
          The up to four CV inputs that you want to mix
        input4 (cv):
          The up to four CV inputs that you want to mix
        auxin1 (cv):
          These auxiliary inputs will be mixed directly into the four outputs output1 …
          output4 and are used for cascading several matrix mixers into one with more
          than four inputs.
        auxin2 (cv):
          These auxiliary inputs will be mixed directly into the four outputs output1 …
          output4 and are used for cascading several matrix mixers into one with more
          than four inputs.
        auxin3 (cv):
          These auxiliary inputs will be mixed directly into the four outputs output1 …
          output4 and are used for cascading several matrix mixers into one with more
          than four inputs.
        auxin4 (cv):
          These auxiliary inputs will be mixed directly into the four outputs output1 …
          output4 and are used for cascading several matrix mixers into one with more
          than four inputs.
        mixmax (fraction):
          If this is 0.0, normal mixing is done (the enabled inputs CVs will be added).
          At a value of 1.0 instead each outputs is the maximum of the enabled inputs.
          Any number in between will create a weighted average between these two values.
        startvalue (integer):
          This input selects in which state the matrix begins life. Also a trigger to
          clear will create that starting state. The following three configurations can
          be selected with startvalue:   0All buttons are cleared. 1The buttons on the
          diagonal are active. 2All buttons are set.   When set to 1, input1 is sent to
          output1, input2 to output2 and so on.
        button11 (gate):
          These four buttons decide, to which of the four outputs input1 is being mixed.
        button12 (gate):
          These four buttons decide, to which of the four outputs input1 is being mixed.
        button13 (gate):
          These four buttons decide, to which of the four outputs input1 is being mixed.
        button14 (gate):
          These four buttons decide, to which of the four outputs input1 is being mixed.
        button21 (gate):
          These four buttons decide, to which of the four outputs input2 is being mixed.
        button22 (gate):
          These four buttons decide, to which of the four outputs input2 is being mixed.
        button23 (gate):
          These four buttons decide, to which of the four outputs input2 is being mixed.
        button24 (gate):
          These four buttons decide, to which of the four outputs input2 is being mixed.
        button31 (gate):
          These four buttons decide, to which of the four outputs input3 is being mixed.
        button32 (gate):
          These four buttons decide, to which of the four outputs input3 is being mixed.
        button33 (gate):
          These four buttons decide, to which of the four outputs input3 is being mixed.
        button34 (gate):
          These four buttons decide, to which of the four outputs input3 is being mixed.
        button41 (gate):
          These four buttons decide, to which of the four outputs input4 is being mixed.
        button42 (gate):
          These four buttons decide, to which of the four outputs input4 is being mixed.
        button43 (gate):
          These four buttons decide, to which of the four outputs input4 is being mixed.
        button44 (gate):
          These four buttons decide, to which of the four outputs input4 is being mixed.
        select (integer):
          The select input allows you to overlay buttons and LEDs with multiple
          functions. If you use this input, the circuit will process the buttons and
          LEDs just as if select has a positive gate signal (usually you will select
          this to 1). Otherwise it won't touch them so they can be used by another
          circuit. Note: even if the circuit is currently not selected, it will
          nevertheless work and process all its other inputs and its  outputs (those
          that do not deal with buttons or LEDs) in a normal way.
        selectat (integer):
          This input makes the select input more flexible. Here you specify at which
          value select should select this circuit. E.g. if selectat is 0, the circuit
          will be active if select is exactly 0 instead of a positive gate signal. In
          some cases this is more conventient.
        preset (integer):
          This is the preset number to save or to load. Note: the first preset has the
          number 0, not 1! For the whole story on presets please refer to page 21.
        loadpreset (trigger):
          A trigger here loads a preset. As a speciality you can use the trigger for
          selecting a preset at the same time.
        savepreset (trigger):
          A trigger here saves a preset.
        clear (trigger):
          A trigger here loads the default start state into the circuit. The presets are
          not affected, unless you use direct preset switching with the preset input and
          without triggers. And that case the current preset is also cleared.
        clearall (trigger):
          A trigger here loads the default start state into the circuit and into all of
          its presets.
        dontsave (gate):
          If you set this to 1, the state of the circuit will not saved to the SD card
          and not loaded from the SD card when the Droid starts.
        output1 (cv):
          The four outputs
        output2 (cv):
          The four outputs
        output3 (cv):
          The four outputs
        output4 (cv):
          The four outputs
        led11 (fraction):
          The LEDs in the buttons button11 …button14
        led12 (fraction):
          The LEDs in the buttons button11 …button14
        led13 (fraction):
          The LEDs in the buttons button11 …button14
        led14 (fraction):
          The LEDs in the buttons button11 …button14
        led21 (fraction):
          The LEDs in the buttons button21 …button24
        led22 (fraction):
          The LEDs in the buttons button21 …button24
        led23 (fraction):
          The LEDs in the buttons button21 …button24
        led24 (fraction):
          The LEDs in the buttons button21 …button24
        led31 (fraction):
          The LEDs in the buttons button31 …button34
        led32 (fraction):
          The LEDs in the buttons button31 …button34
        led33 (fraction):
          The LEDs in the buttons button31 …button34
        led34 (fraction):
          The LEDs in the buttons button31 …button34
        led41 (fraction):
          The LEDs in the buttons button41 …button44
        led42 (fraction):
          The LEDs in the buttons button41 …button44
        led43 (fraction):
          The LEDs in the buttons button41 …button44
        led44 (fraction):
          The LEDs in the buttons button41 …button44
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 176
    input1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    auxin1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    auxin2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    auxin3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    auxin4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    mixmax: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    startvalue: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    button11: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button12: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button13: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button14: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button21: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button22: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button23: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button24: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button31: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button32: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button33: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button34: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button41: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button42: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button43: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    button44: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    select: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    selectat: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    preset: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    loadpreset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    savepreset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    clear: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    clearall: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    dontsave: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    led11: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led12: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led13: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led14: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led21: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led22: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led23: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led24: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led31: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led32: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led33: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led34: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led41: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led42: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led43: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    led44: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )



@dataclass
class Mixer(DroidCircuit):
    """CV mixer.

    Args:
        input1 (cv):
          1 ... 8 mixing input
        input2 (cv):
          1 ... 8 mixing input
        input3 (cv):
          1 ... 8 mixing input
        input4 (cv):
          1 ... 8 mixing input
        input5 (cv):
          1 ... 8 mixing input
        input6 (cv):
          1 ... 8 mixing input
        input7 (cv):
          1 ... 8 mixing input
        input8 (cv):
          1 ... 8 mixing input
        output (cv):
          Sum of all patched inputs
        maximum (cv):
          Maximum of all patched inputs of this circuit. This can e.g. be used for
          mixing together the envelopes from several sequencer tracks without making
          them “louder” or distorting them when two sequencers play a note at the same
          time.
        minimum (cv):
          Minimum of all patched inputs of this circuit.
        average (cv):
          Average of all patched inputs of this circuit.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 48
    input1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    maximum: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    minimum: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    average: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Quantizer(DroidCircuit):
    """Non-musical quantizer.

    Args:
        input (cv):
          Patch the unquantized input voltage here
        trigger (trigger):
          This jack is optional. If you patch it, the quantizer will work in triggered
          mode. Here the output pitch is always frozen until the next trigger happens.
        steps (integer):
          Number of steps that one Volt should be divided in. The default is 12 and will
          quantize the input voltage to semitones. The number of steps is related to a
          value of 1 V which means 0.1. It is allowed to use a fractional number here.
          E.g. the value 1.2 will quantize to 12 steps per 10 V (which means 12 steps
          per 1.0, which can make sense. A value of 0 (or lower) will basically mean an
          infinite number of steps and thus practically disable quantization.
        bypass (gate):
          If you set this gate input to 1 then quantization is bypassed and the input
          voltage is directly copied to the output.
        direction (integer):
          This parameter selects which value is chosen if the input value lies between
          two allowed quantized values (which is almost always the case). The default is
          to choose the nearest value.  0quantized down 1quantize to nearest allowed
          value 2quantize up
        output (stepped):
          Here comes your quantized output voltage
        changed (trigger):
          Whenever the quantization changes to a new output value a trigger with the
          duration 10 ms is output here. No trigger is output in bypass mode.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 48
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    trigger: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    steps: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    bypass: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    direction: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_stepped(ramsize=0)
    )
    changed: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )



@dataclass
class Queue(DroidCircuit):
    """Clocked CV shift register.

    Args:
        input (cv):
          This CV will be pushed into the first cell of the shift register whenever a
          clock occurs.
        clock (trigger):
          Each clock signal at this jack will move the CV content from every cell of the
          shift register to the next cell. The CV in the last cell will be dropped.
        outputpos1 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        outputpos2 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        outputpos3 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        outputpos4 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        outputpos5 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        outputpos6 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        outputpos7 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        outputpos8 (integer):
          Specifies the position of each of the eight outputs – i.e. which cell of the
          shift register it should output. Allowed are values from 1 up to 64. These
          jacks defaults to 1, 2, ... 8, so if you do not wire them the eight outputs
          reflect the first eight positions of the shift register.
        output1 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        output2 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        output3 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        output4 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        output5 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        output6 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        output7 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        output8 (cv):
          Eight outputs for eight different positions of the register. If you do not
          wire outputpos1 ... outputpos8, these outputs show the content of the 1, 2,
          ... 8 cell.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 312
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    outputpos1: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    outputpos2: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    outputpos3: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    outputpos4: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    outputpos5: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    outputpos6: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    outputpos7: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    outputpos8: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Recorder(DroidCircuit):
    """Record and playback CVs und gates.

    Args:
        playbutton (trigger):
          A trigger here starts or restarts the playback.
        recordbutton (trigger):
          A trigger here starts or stops the recording.
        stopbutton (trigger):
          A trigger here stops and ongoing playback or recording.
        loop (gate):
          Set this to 1 to enable loop mode. In loop mode the playback is restarted
          immediately when it's finished.
        pause (gate):
          While this is 1, playback or recording is halted (the tape stops moving for
          the while).
        mode (integer):
          Using this input is an alternative to using the three button inputs. If you
          patch mode, the three buttons (and LED outputs) are ignored. Instead you set
          the mode with this input:   0Idle / stopped 1Playback 2Recording   Since you
          set the mode from “outside”, the recorder cannot switch it by itself. So if
          the mode is set to 1 (playback) and the playback is finished, it stays in
          playback mode and continues outputting the last sample.
        playbackspeed (cv):
          Sets the speed of the tape during playback. 1 is normal speed, 0.5 half speed,
          2 double speed, and so on. Negative speeds are allowed an move the tape
          backwards. The speed 0 stops the tape.
        scrub (gate):
          If 1 enables scrubbing. Now the outputs reflect the tape position that is set
          with scrubposition.
        scrubposition (fraction):
          Position of the tape to play when scrubbing is enabled.
        trimstart (fraction):
          Omits a fraction of the recording at the beginning during playback and
          scrubbing. 0.1 omits the first 10%.
        trimend (fraction):
          Omits a fraction of the recording at the end during playback and scrubbing.
          0.8 omits the last 20% (not 80%!).
        cvin (cv):
          Continous input CV
        numberin (integer):
          Discrete input number in the range 0 ... 255
        gatein1 (cv):
          Input gates
        gatein2 (cv):
          Input gates
        gatein3 (cv):
          Input gates
        gatein4 (cv):
          Input gates
        gatein5 (cv):
          Input gates
        gatein6 (cv):
          Input gates
        gatein7 (cv):
          Input gates
        gatein8 (cv):
          Input gates
        clock (trigger):
          If you use this clock input, all time inputs are measured in clock ticks
          instead of seconds.
        sample (gate):
          If you patch this input, “triggered” mode is enabled. In this mode, the
          virtual tape just records a new CV on each trigger at sample. So it just
          records stepped CVs, no slopes and no CV changes between the triggers.
        timewindow (cv):
          When in triggered mode, this optional parameter helps tackling a problem that
          many hardware sequencers show: often their pitch CV is not at its final
          destination value at the time their gate is being output. Often you see a very
          short “slew” ramp of say 5 ms after the gate. During that time the pitch CV
          moves from its former to the new value.  Now if you trigger the cvtape circuit
          with the sequencer's gate you will essentially sample the previous pitch CV
          instead of the new one. Or maybe something in between.  The timewindow
          parameter configures a short time window after the trigger to trigger. During
          that time period the tape will constantly adapt the last sample to a changed
          input CV. When that time is over, the input is finally frozen on the tape.
          The timewindow parameter is in seconds. So when you set timewindow to say
          0.005 (which means 5 ms), you give the input CV 5 ms time for settling to its
          final value after a trigger to sample before freezing it.
        bypass (gate):
          Setting bypass to on copies the input signals directly to the outputs,
          regardless of any other stuff going on.
        save (trigger):
          Send a trigger here to save the current contents of the tape to a file on the
          SD card. The filename is tapeXXXX.bin, where XXXX is replaced by the number
          set by filenumber.
        load (trigger):
          Send a trigger here to load a previously saved file into the tape. Use
          filenumber so specify which file to load.
        filenumber (integer):
          Number of the file to load or save. The range is 0 - 9999. If filenumber is
          123, the name on the SD card is tape0123.bin. These files are shared between
          all recorder and delay circuits.
        select (integer):
          The select input allows you to overlay buttons and LEDs with multiple
          functions. If you use this input, the circuit will process the buttons and
          LEDs just as if select has a positive gate signal (usually you will select
          this to 1). Otherwise it won't touch them so they can be used by another
          circuit. Note: even if the circuit is currently not selected, it will
          nevertheless work and process all its other inputs and its  outputs (those
          that do not deal with buttons or LEDs) in a normal way.
        selectat (integer):
          This input makes the select input more flexible. Here you specify at which
          value select should select this circuit. E.g. if selectat is 0, the circuit
          will be active if select is exactly 0 instead of a positive gate signal. In
          some cases this is more conventient.
        recordled (gate):
          Is 1 during recordings.
        playled (gate):
          Is 1 during playback or scrubbing.
        stopled (gate):
          Is 1 when no playback, recording or scrubbing is going on.
        cvout (cv):
          Output of the continous input CV
        numberout (integer):
          Output of the discrete number
        gateout1 (gate):
          Output of the gates
        gateout2 (gate):
          Output of the gates
        gateout3 (gate):
          Output of the gates
        gateout4 (gate):
          Output of the gates
        gateout5 (gate):
          Output of the gates
        gateout6 (gate):
          Output of the gates
        gateout7 (gate):
          Output of the gates
        gateout8 (gate):
          Output of the gates
        overflow (gate):
          When the internal memory of the tape is exceeded and data got lost, this gate
          goes to 1 for 0.5 seconds. If you are suspecting this situation, you can wire
          this output to an LED and observe the memory status that way.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 1712
    playbutton: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    recordbutton: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    stopbutton: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    loop: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    pause: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    mode: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    playbackspeed: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    scrub: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    scrubposition: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    trimstart: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    trimend: Optional[str] = field(
            default=None,
            metadata=ctype.type_fraction(ramsize=0)
    )
    cvin: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    numberin: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    gatein1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatein8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    sample: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    timewindow: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    bypass: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    save: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    load: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    filenumber: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    select: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    selectat: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    recordled: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    playled: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    stopled: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    cvout: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    numberout: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    gateout1: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout2: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout3: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout4: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout5: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout6: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout7: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gateout8: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    overflow: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )



@dataclass
class Sample(DroidCircuit):
    """Sample & Hold Circuit.

    Args:
        input (cv):
          Input signal to be sampled
        sample (trigger):
          A positive trigger here will read the current value from input and store it
          internally.
        gate (gate):
          This is an alternative way of making the circuit take a sample from the input.
          Here it is sampling all the time while the gate is high. In that way it is a
          bit like bypass. But as soon as the gate goes low again, the output sticks to
          the last sample value just before that.
        timewindow (cv):
          This optional parameter helps tackling a problem that many (non-analog)
          sequencers show: often their pitch CV is not at its final destination value at
          the time their gate is being output. Often you see a very short “slew” ramp of
          say 5 ms after the gate. During that time the pitch CV moves from its former
          to the new value.  Now if you trigger the sample circuit with the sequencer's
          gate you will essentially sample the previous pitch CV instead of the new one.
          Or maybe something in between.  Now the timewindow parameter introduces a
          short time window after the sample trigger. During that time period the sample
          & hold circuit will constantly adapt to a changed input CV (is essentially in
          bypass mode). When that time is over, the input is finally frozen.  The
          timewindow parameter is in seconds. So when you set timewindow to say 0.005
          (which means 5 ms), you give the input CV 5 ms time for settling to its final
          value after a trigger to sample before freezing it.
        bypass (gate):
          While this gate input is high, the circuit is bypassed and input is copied to
          output.
        output (cv):
          The most recently sampled value is sent here.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 40
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    sample: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    gate: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    timewindow: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    bypass: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Slew(DroidCircuit):
    """Slew limiter.

    Args:
        input (cv):
          Wire the CV that you wish to slew limit here.
        slew (cv):
          This controls the slew rate. A value of 0.0 disables slew limiting. The output
          immediately follows the input without any delay. A value of for example 2.0 in
          linear mode means that 2.0 seconds are needed for a change of 1 V (which is a
          value of 0.1 or one octave if used as pitch). In the other two modes the slew
          time is tuned to sound similar. Negative values of this parameter are treated
          as 0.0.
        slewup (cv):
          This allows a special handling when the voltage moves upwards. The slew
          limiting for upwards is slew multiplied with slewup. Since slew defaults to
          1.0 you can just use slewup and slewdown if you want to control both
          directions separately.
        slewdown (cv):
          Sets the slew rate for downwards movement.
        gate (gate):
          If this jack is patched, the slew limiting is only active while this gate is
          high. Otherwise it's like setting the slew parameter to zero.
        exponential (cv):
          Output for the resulting CV with the exponential (classical) slew algorithm
          applied
        linear (cv):
          Output for linear slew limiting
        scurve (cv):
          Output with the slew limitation according to the S-curve algorithm.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 48
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    slew: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    slewup: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    slewdown: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gate: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    exponential: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    linear: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    scurve: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Switch(DroidCircuit):
    """Adressable/clockable switch.

    Args:
        input1 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input2 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input3 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input4 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input5 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input6 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input7 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input8 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input9 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input10 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input11 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input12 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input13 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input14 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input15 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        input16 (cv):
          1 ... 16 input. Use these inputs in order and don't leave gaps.
        forward (trigger):
          If a trigger or gate is received here, the switch adds one to the current
          internal switch offset. So every output moves to the next input and every
          input moves to the previous output.
        backward (trigger):
          Similar then forward, but switches backwards
        reset (trigger):
          Resets the switch to its initial position. Assuming offset is at 0, input1 is
          connected to output1, input2 to output2 etc.  If reset and a trigger at
          forward / backward happen at the same time (within 5 ms), the reset will win
          and the switch is being reset to offset 0. This avoids problems with unprecise
          timing of external sequencers.
        offset (integer):
          This allows CV addressable switching. The number read here is being used a
          shifting offset and is always added to the internal offset. For example if you
          send 5 here, it is like you have triggered forward five times after the last
          reset. Please note, then 5 would mean 50 Volts, not 5 Volts. So if you patch
          an external CV like I1 here, you probably want to multiply with some useful
          number.
        output1 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output2 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output3 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output4 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output5 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output6 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output7 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output8 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output9 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output10 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output11 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output12 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output13 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output14 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output15 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        output16 (cv):
          1 ... 16 output. Use these outputs in order and don't leave gaps.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 104
    input1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input9: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input10: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input11: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input12: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input13: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input14: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input15: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    input16: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    forward: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    backward: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    reset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    offset: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output9: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output10: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output11: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output12: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output13: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output14: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output15: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output16: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )






"""Circuit generation types."""

import keyword

from collections import defaultdict
from dataclasses import dataclass, field
from typing import Dict, List

from .formatting import strip_latex, format_synopsis


@dataclass
class Parameter:
    """Parameter."""

    name: str
    type: str
    description: str
    ramsize: int
    direction: str

    def __post_init__(self) -> None:
        """Reformat name if it collides with python."""
        if keyword.iskeyword(self.name):
            self.name += "_"
        description = " ".join(self.description.splitlines())
        self.description = strip_latex(description)

    @property
    def synopsis(self) -> str:
        """Return a synopsis for the docstring."""
        return format_synopsis(self.description)


@dataclass
class Circuit:
    """Circuit container."""

    name: str
    title: str
    category: str
    parameters: List[Parameter] = field(default_factory=list)
    ramsize: int = 0

    def __post_init__(self) -> None:
        """Remove latex from title for good meassure."""
        self.title = strip_latex(self.title)


@dataclass
class CircuitCollection:
    """Enclosing dataclass for a collection of circuits."""

    droid_version: str
    circuits: List[Circuit] = field(default_factory=list)

    def split_categories(self) -> Dict[str, "CircuitCollection"]:
        """Generate an instance of self for each of the circuit categories."""
        c_categories: Dict[str, List[Circuit]] = defaultdict(list)
        for circuit in self.circuits:
            c_categories[circuit.category].append(circuit)

        return {
            category: CircuitCollection(self.droid_version, c_circuits)
            for category, c_circuits in c_categories.items()
        }



"""Patch Generator module.

.. include:: ../README.md

# Public API

"""

from .metapatch import PatchGenerator
from .options import option, preset
from . import circuits
from .circuits.base import transform, DroidCircuit

__docformat__ = "google"

__all__ = [
    "PatchGenerator",
    "option",
    "preset",
    "circuits",
    "transform",
    "DroidCircuit",
]



"""DROID circuits. These circuits are auto-generated from circuits.json."""

from dataclasses import dataclass, field
from typing import Optional

from metapatch.circuits.base import DroidCircuit
from metapatch.circuits import circuit_types as ctype


__droid_version__ = "magenta-3"


@dataclass
class Bernoulli(DroidCircuit):
    """Random gate distributor.

    Args:
        input (gate):
          Send gate or trigger signals here.
        distribution (bipolar):
          This controls the probability of a gate to be forwarded to output1. A value of
          0.5 means 50%.
        output1 (gate):
          Gates from input are forwarded here if the random decision was in favour of
          output 1.
        output2 (gate):
          Gates from input are forwarded here if the random decision was in favour of
          output 2.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 32
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    distribution: Optional[str] = field(
            default=None,
            metadata=ctype.type_bipolar(ramsize=0)
    )
    output1: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    output2: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )



@dataclass
class Burst(DroidCircuit):
    """Generate burst of pulses.

    Args:
        rate (cv):
          Frequency control: The default frequency of the burst rate is 1 Hz (one
          trigger per second or 60 BPM if you like). Each volt doubles the frequency. So
          an input of 1 V (a number of 0.1) speeds up to two triggers per second
          (120 BPM), 2 V (0.2) creates triggers at 4 Hz (240 BPM) and so on. On the
          other hand negative voltages reduce the speed, so -1 V (-0.1) will give 0.5 Hz
          (30 BPM) and so on.
        taptempo (trigger):
          Feed a reference clock here and the burst will run at the speed of that clock
          – albeit optionally modified by rate. Please see page 23 for details on using
          taptempo inputs.
        hz (cv):
          Set the frequency in Hz directly by setting a number here. This is exclusive
          to taptempo, but will work in combination with rate.
        trigger (trigger):
          Send a trigger here in order to start the bursts
        reset (trigger):
          Send a trigger here to immediately stop any ongoing burst.
        count (integer):
          Number of triggers to send in one burst.
        skip (integer):
          Number of time slots to wait before starting with the burst.
        output (trigger):
          The triggers are output here.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 40
    rate: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    taptempo: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    hz: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    trigger: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    reset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    count: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    skip: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )



@dataclass
class Clocktool(DroidCircuit):
    """Clock divider / multiplier / shifter.

    Args:
        clock (trigger):
          Patch a steady clock here for this circuit to be of any use
        reset (trigger):
          A trigger here resets the internal counters. This is useful if you use the
          clock divider and want to restart the internal counting from 0, in order to
          align the clock divider with some external sequencers or the like
        divide (integer):
          Number to divide the clock through. This will be rounded to the nearest
          integer number. Note: if you want to use an external CV then you need to
          multiply that with some useful number, since otherwise you will get a number
          between 0 and 1 which is not useful at all. Remember: 10 V translates to a
          number of 1.
        multiply (integer):
          Number to multiply the clock with. Same considerations hold as for divide.
        dutycycle (bipolar):
          Output duty cycle of the clock – which is essentially a square wave – in a
          range from 0.0 to 1.0 or 0% to 100%. If you don't patch anything here, the
          length of the trigger output pulses will be 10 ms ('s standard trigger
          duration).
        gatelength (cv):
          This jack is alternative to dutycycle and will override it if it is used.
          It sets the length of each output pulse to a fixed value that is independent
          of the incoming clock. A value of 0.5 (a CV of 5 volts) translates into a gate
          length of 0.5 seconds.
        delay (cv):
          This CV allows you to shift the input clock beat around in time. A value of
          0.1 will delay each beat by 10% of a clock cycle. A value of -0.1 is also
          allowed and shifts the beat 10% ahead.  For an unmodulated delay -0.1 and 0.9
          is just the same, because the output clock will have the same relation to the
          input clock. But if you modify the delay from 0.0 to 0.9, the next tick will
          be delayed by 90% of one cycle, where is a modification from 0.0 to -0.1 will
          play the next tick by 10% earlier.
        output (gate):
          Here comes the modified clock
        inputpitch (cv):
          Experimental output that outputs a representation of the input clock's pitch
          on a 1V/octave base, based on the reference of 60 BPM (1 Hz). This means that
          an input clock of 120 BPM will output 1V (a value of 0.1), since 120 BPM it is
          one octave higher than 60 BPM. If you feed that value to the rate input of an
          LFO you get that running at exactly the same speed (not in the same phase,
          however).
        outputpitch (cv):
          Same for the modified output clock
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 96
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    reset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    divide: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    multiply: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    dutycycle: Optional[str] = field(
            default=None,
            metadata=ctype.type_bipolar(ramsize=0)
    )
    gatelength: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    delay: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    inputpitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    outputpitch: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )



@dataclass
class Flipflop(DroidCircuit):
    """Simple flip flop.

    Args:
        toggle (trigger):
          A trigger here inverts the state of the flip flop. It changes 0 to 1 and 1 to
          0.
        set (trigger):
          Sets the flip flop to 1.
        reset (trigger):
          Sets the flip flop to 0.
        clear (trigger):
          Sets the flip flop to the value defined by startvalue.
        startvalue (gate):
          The flip flop starts its live with this value. Also clear will set the flip
          flop to this value.
        load (trigger):
          Loads the value into the flip flop that's defined with loadvalue.
        loadvalue (gate):
          Value to set the flip flop to, when load is triggered.
        output (gate):
          Outputs the current value of the flip flop: either 0 or 1.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 40
    toggle: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    set: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    reset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    clear: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    startvalue: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    load: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    loadvalue: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )



@dataclass
class Gatetool(DroidCircuit):
    """Operate on triggers and gates, modify gatelength.

    Args:
        inputgate (gate):
          Input gate. Use this if the length of the input gate is relevant.
        inputtrigger (trigger):
          Input trigger. Use this if the length of the input gate should be ignored.
        inputedge (gate):
          Input edge: Use this if every low/high or high/low transition should count as
          a trigger.
        gatelength (cv):
          Sets the length of the gate of outputgate in seconds. If you use taptempo the
          length is in fractions of a clock tick, instead.
        gatestretch (cv):
          Makes the output gate longer than the input gate by the given percentage. This
          parameter is ignored if gatelength is used.
        mingatelength (cv):
          Defines a minimum length of the output gate in seconds or clock ticks.
        maxgatelength (cv):
          Defines a maximum length of the output gate in seconds or clock ticks.
        taptempo (trigger):
          If you patch a reference clock here, gatelength, mingatelength and
          maxgatelength are fractions of one clock tick, not seconds anymore. Please see
          page 23 for details on using taptempo inputs.
        outputgate (gate):
          Outputs a gate with controllable length for every gate, trigger or edge event.
        outputtrigger (trigger):
          Outputs a 10 ms trigger for every gate, trigger or edge event.
        outputedge (gate):
          Toggle between 0 and 1 at every gate, trigger or edge event.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 56
    inputgate: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    inputtrigger: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    inputedge: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    gatelength: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatestretch: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    mingatelength: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    maxgatelength: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    taptempo: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    outputgate: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    outputtrigger: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    outputedge: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )



@dataclass
class Once(DroidCircuit):
    """Output one trigger after the Droid has started.

    Args:
        delay (cv):
          Set a delay in seconds after the Droid's start before the trigger triggers.
          Note: the default value is 10 ms, not zero. This allows all attached
          controllers to have sent at least one update before and the real pot values
          etc. are available at the circuits.
        onlycoldstart (gate):
          If you set this input to 1, once just sends a trigger after a cold start,
          only. A cold start means that the Droid has been powered up. Pressing the
          button for loading a new patch and does a warm start.
        trigger (trigger):
          The trigger is output here.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 24
    delay: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    onlycoldstart: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    trigger: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )



@dataclass
class Timing(DroidCircuit):
    """Shuffle/swing and complex timing generator.

    Args:
        clock (trigger):
          Patch a steady clock here for this circuit to be of any use
        reset (trigger):
          A trigger here resets the internal step counter and restart at step 1.
        timing1 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        timing2 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        timing3 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        timing4 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        timing5 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        timing6 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        timing7 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        timing8 (cv):
          Specifies a relative timing for each step in relation to the input clock. A
          timing of 0.3 will shift the respective beat 30% of a clock cycle behind,
          while -0.3 will make it 30% early.  The timing values are clipped into the
          range -0.9999 …0.9999.
        output (trigger):
          Here comes the modified output clock
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 56
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    reset: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    timing1: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    timing2: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    timing3: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    timing4: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    timing5: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    timing6: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    timing7: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    timing8: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )



@dataclass
class Triggerdelay(DroidCircuit):
    """Trigger Delay with multi tap and optional clocking.

    Args:
        input (gate):
          Patch triggers or gates to be delayed here.
        delay (cv):
          Amount of time the incoming triggers are being delayed. When clock is patched,
          this is in relation to one clock cycle, so a delay of 4 will delay the input
          pattern by exactly 4 beats. Fractions are allowed also. If clock is not
          patched, this parameter is in seconds. So for example in order to delay by
          100 ms you need a delay of 0.1.
        gatelength (cv):
          Unless you patch this jack the length of the output gates is exactly the
          length of the input gates. By use of this parameter you override that length
          and set a fixed length in seconds – or if clock is being used – in clock
          cycles.
        repeats (integer):
          Number of times the delayed trigger is being repeated. Each further repetition
          is with the same delay.
        mute (gate):
          A high gate signal suppresses any further output gates. However, the current
          gate is finished normally.
        clock (trigger):
          When you patch this input, the trigger delay runs in clocked mode. In this
          mode delay is relative to one clock cycle. I.e. a delay if 0.5 will delay the
          trigger by half a clock cycle. The same holds for gatelength. That is measured
          in clock cycles, too.
        output (gate):
          Outputs the delayed triggers/gates, while keeping the gate length – unless you
          have changed that
        overflow (gate):
          Whenever there are more input triggers than this circuit can keep memory of,
          this output outputs a gate of 0.5 sec length. You can wire this to an LED in
          order to know when this happens.
        comment: Add a comment in the droid ini file.

    """

    __ramsize__ = 248
    input: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    delay: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    gatelength: Optional[str] = field(
            default=None,
            metadata=ctype.type_cv(ramsize=0)
    )
    repeats: Optional[str] = field(
            default=None,
            metadata=ctype.type_integer(ramsize=0)
    )
    mute: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    clock: Optional[str] = field(
            default=None,
            metadata=ctype.type_trigger(ramsize=0)
    )
    output: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )
    overflow: Optional[str] = field(
            default=None,
            metadata=ctype.type_gate(ramsize=0)
    )






"""DROID circuits module.

These circuits are auto-generated from circuits.json for droid firmware version
'magenta-3'.


.. include:: README.md
"""

__docformat__ = "google"


from .logic import (
    Adc,
    Compare,
    Copy,
    Dac,
    Explin,
    Ifequal,
    Logic,
    Math,
    Multicompare,
    Select,
)
from .sequencing import (
    Algoquencer,
    Arpeggio,
    Encoquencer,
    Euklid,
    Motoquencer,
    Polytool,
    Sequencer,
)
from .clock import (
    Bernoulli,
    Burst,
    Clocktool,
    Flipflop,
    Gatetool,
    Once,
    Timing,
    Triggerdelay,
)
from .ui import (
    Button,
    Buttongroup,
    Encoderbank,
    Encoder,
    Faderbank,
    Fadermatrix,
    Motorfader,
    Notebuttons,
    Nudge,
    Pot,
    Unusedfaders,
)
from .pitch import (
    Calibrator,
    Chord,
    Detune,
    Fold,
    Minifonion,
    Octave,
    Sinfonionlink,
    Superjust,
    Vcotuner,
)
from .cv import (
    Case,
    Crossfader,
    Cvlooper,
    Delay,
    Matrixmixer,
    Mixer,
    Quantizer,
    Queue,
    Recorder,
    Sample,
    Slew,
    Switch,
)
from .modulation import (
    Contour,
    Lfo,
    Random,
    Spring,
    Transient,
)
from .other import (
    Droid,
    Outputcalibrator,
)
from .midi import (
    Firefacecontrol,
    Midifileplayer,
    Midiin,
    Midiout,
    Midithrough,
)
from .deprecated import (
    Fourstatebutton,
    Notchedpot,
    Switchedpot,
    Togglebutton,
)


__all__ = (
    "Adc",
    "Compare",
    "Copy",
    "Dac",
    "Explin",
    "Ifequal",
    "Logic",
    "Math",
    "Multicompare",
    "Select",
    "Algoquencer",
    "Arpeggio",
    "Encoquencer",
    "Euklid",
    "Motoquencer",
    "Polytool",
    "Sequencer",
    "Bernoulli",
    "Burst",
    "Clocktool",
    "Flipflop",
    "Gatetool",
    "Once",
    "Timing",
    "Triggerdelay",
    "Button",
    "Buttongroup",
    "Encoderbank",
    "Encoder",
    "Faderbank",
    "Fadermatrix",
    "Motorfader",
    "Notebuttons",
    "Nudge",
    "Pot",
    "Unusedfaders",
    "Calibrator",
    "Chord",
    "Detune",
    "Fold",
    "Minifonion",
    "Octave",
    "Sinfonionlink",
    "Superjust",
    "Vcotuner",
    "Case",
    "Crossfader",
    "Cvlooper",
    "Delay",
    "Matrixmixer",
    "Mixer",
    "Quantizer",
    "Queue",
    "Recorder",
    "Sample",
    "Slew",
    "Switch",
    "Contour",
    "Lfo",
    "Random",
    "Spring",
    "Transient",
    "Droid",
    "Outputcalibrator",
    "Firefacecontrol",
    "Midifileplayer",
    "Midiin",
    "Midiout",
    "Midithrough",
    "Fourstatebutton",
    "Notchedpot",
    "Switchedpot",
    "Togglebutton",
)


#!/usr/bin/env python3
"""This is a small helper script that can be used to speed up some of the patch building."""
import argparse
import sys
import os
from typing import TextIO


import metapatch.factory


DEFAULT_TITLE = "Draft Patch Generator"
DEFAULT_DESCRIPTION = "A short description"
DEFAULT_CLASSNAME = "PatchGenerator"


def get_patch_stdin() -> str:
    """Read a patch from stdin."""
    buffer = []
    empty_lines = 0
    while True:
        line = sys.stdin.readline()
        if line == "\n":
            empty_lines += 1
            if empty_lines == 2:
                break
            else:
                buffer.append(line.rstrip("\n"))
        elif line == "":
            break
        else:
            buffer.append(line.rstrip("\n"))

    return "\n".join(buffer)


def cli() -> argparse.Namespace:
    """Parse cli."""
    parser = argparse.ArgumentParser(description="Metapatch helper script.")

    subparsers = parser.add_subparsers(help="actions", metavar="action", dest="action")

    convert_parser = subparsers.add_parser(
        "convert", help="Convert (partial) droid patches to python statements."
    )

    convert_parser.add_argument(
        "infile",
        nargs="?",
        type=argparse.FileType("r", encoding="utf-8"),
        default=sys.stdin,
        help="Input file. Defaults to stdin.",
    )
    boilerplate_parser = subparsers.add_parser(
        "generate", help="Generate a patch generator from a droid patch."
    )
    boilerplate_parser.add_argument(
        "infile", type=argparse.FileType("r", encoding="utf-8")
    )
    boilerplate_parser.add_argument(
        "outfile",
        nargs="?",
        type=argparse.FileType("w", encoding="utf-8"),
        default=sys.stdout,
        help="Output file. Defaults to stdout.",
    )

    boilerplate_parser.add_argument(
        "--title", help="Title of your patch generator.", default=DEFAULT_TITLE
    )
    boilerplate_parser.add_argument(
        "--description",
        help="Description of your patch generator.",
        default=DEFAULT_DESCRIPTION,
    )
    boilerplate_parser.add_argument(
        "--classname",
        help="Name of the python class for the patch generator.",
        default=DEFAULT_CLASSNAME,
    )

    parser.epilog = (
        "If called without any arguments, you may input one or more droid circuits, "
        "and get the corresponding python code back."
    )
    args = parser.parse_args()
    if not args.action:
        if not sys.stdin.isatty():
            parser.exit(1)
        else:
            args.action = "convert"
            args.infile = sys.stdin

    return args


def cli_convert(infile: TextIO) -> None:
    """Convert on the CLI."""
    if infile.isatty():
        print("Paste a circuit configuration and finish with ctrl-d.\n")
    buffer = infile.read()

    print(metapatch.factory.generate_snippet(buffer))


def cli_boilerplate(args: argparse.Namespace) -> None:
    """Generate boilerplate."""
    buffer = args.infile.read()
    output = metapatch.factory.generate_boilerplate(
        buffer, args.title, args.description, args.classname
    )
    args.outfile.write(output + os.linesep)


def main() -> None:
    """Main function."""
    args = cli()

    if args.action == "convert":
        cli_convert(args.infile)
    elif args.action == "generate":
        cli_boilerplate(args)
    else:
        print(f"Unsupported action: {args.action}")


if __name__ == "__main__":
    main()



"""Circuit generation."""

from pathlib import Path

from .parsing import load_circuits
from .writer import write_circuits


def generate(filename: str, output_directory: str) -> None:
    """Generate circuits."""
    collection = load_circuits(Path(filename))
    write_circuits(collection, Path(output_directory))


__all__ = ["generate"]
